package rpc

import (
	"context"

	"git.loafle.net/commons_go/rpc"
	crcwf "git.loafle.net/commons_go/rpc/connection/websocket/fasthttp"
	"git.loafle.net/commons_go/rpc/gateway"
	"git.loafle.net/commons_go/rpc/protocol"
	"git.loafle.net/commons_go/rpc/protocol/json"
	oogw "git.loafle.net/overflow/overflow_gateway_websocket"
	"git.loafle.net/overflow/overflow_gateway_websocket/external/grpc"
	"google.golang.org/grpc/metadata"
)

func newGatewayRPCServletHandler() GatewayRPCServletHandler {
	sh := &GatewayRPCServletHandlers{}
	sh.RegisterCodec(json.Name, json.NewServerCodec())
	return sh
}

type GatewayRPCServletHandlers struct {
	gateway.ServletHandlers
	rpcIO crcwf.ServletHandlers
}

func (sh *GatewayRPCServletHandlers) GetRequest(servletCTX rpc.ServletContext, codec protocol.ServerCodec, conn interface{}) (protocol.ServerRequestCodec, error) {
	return sh.rpcIO.GetRequest(servletCTX, codec, conn)
}

func (sh *GatewayRPCServletHandlers) SendResponse(servletCTX rpc.ServletContext, conn interface{}, requestCodec protocol.ServerRequestCodec, result interface{}, err error) error {
	return sh.rpcIO.SendResponse(servletCTX, conn, requestCodec, result, err)
}

func (sh *GatewayRPCServletHandlers) SendNotification(servletCTX rpc.ServletContext, conn interface{}, codec protocol.ServerCodec, method string, args ...interface{}) error {
	return sh.rpcIO.SendNotification(servletCTX, conn, codec, method, args...)
}

func (sh *GatewayRPCServletHandlers) Invoke(servletCTX rpc.ServletContext, requestCodec protocol.RegistryCodec) (result interface{}, err error) {
	md := metadata.Pairs(oogw.GRPCUserIDKey, servletCTX.GetAttribute(oogw.SocketIDKey).(string))
	grpcCTX := metadata.NewOutgoingContext(context.Background(), md)

	params, err := requestCodec.Params()
	if nil != err {
		return nil, err
	}

	r, err := grpc.Exec(grpcCTX, requestCodec.Method(), params)
	if err != nil {
		return nil, err
	}

	return r, nil
}

func (sh *GatewayRPCServletHandlers) Validate() {
	sh.ServletHandlers.Validate()

}
